__author__ = 'Managed by Q, Inc.'
__author_email__ = 'open-source@managedbyq.com'
__description__ = 'MBQ Service Client'
__license__ = 'Apache 2.0'
__title__ = 'mbq.client'
__url__ = 'https://github.com/managedbyq/mbq.client'
__version__ = '1.1.0'
